from django.apps import AppConfig


class DjangoInstagramConfig(AppConfig):
    default_auto_field = "django.db.models.BigAutoField"
    name = "instagram_sync.contrib.django"
    label = "instagram_sync"



import asyncio
import io
from dataclasses import dataclass, fields
from functools import cached_property
from typing import Optional

import requests
from core.settings import DEFAULT_CHUNK_SIZE
from PIL import Image as PILImage
from PIL import UnidentifiedImageError


class IGMediaBase:
    media_url: str

    @classmethod
    def from_dict(cls, dict_):
        class_fields = {f.name for f in fields(cls)}
        return cls(**{k: v for k, v in dict_.items() if k in class_fields})


@dataclass
class IGMediaData:
    media_url: str
    caption: str
    timestamp: str
    permalink: str
    children: Optional[IGMediaBase] = None

    @classmethod
    def from_dict(cls, dict_):
        class_fields = {f.name for f in fields(cls)}
        return cls(**{k: v for k, v in dict_.items() if k in class_fields})


class IGMediaObject:
    def __init__(self, data):
        self.__media_data = IGMediaData.from_dict(data)
        media_data = self.__media_data
        response = requests.get(media_data.media_url)

        self.__response_content = response.content

    @cached_property
    def __pil_image(self):
        pil_image = None
        try:
            content_file = io.BytesIO(self.bytes)
            pil_img = PILImage.open(content_file)
        except UnidentifiedImageError:
            print(f"error opening {self.graph.media_url}")
            return

        return pil_img

    @property
    def width(self):
        return self.__pil_image.width if self.__pil_image else None

    @property
    def height(self):
        return self.__pil_image.height if self.__pil_image else None

    @property
    def graph(self):
        return self.__media_data

    @property
    def bytes(self):
        return self.__response_content


class IGMediaCollection:
    """Class for keeping track of a collection of IGMedia objects."""

    def __init__(self, data: list[dict], chunk_size=DEFAULT_CHUNK_SIZE, *args, **kwargs):
        self.__collection = []
        asyncio.run(self.__populate_collection(data, chunk_size=chunk_size))

    async def __populate_collection(self, data, chunk_size=DEFAULT_CHUNK_SIZE):
        if len(data) > chunk_size:
            print("breaking into chunks")
            page = 1
            upper_bound = (len(data) // chunk_size) + 1
            while page <= upper_bound:
                print(f"chunk {page}")
                start_idx = (page - 1) * chunk_size
                end_idx = (page) * chunk_size
                page += 1
                downloaded_media = await asyncio.gather(
                    *[self.__get_media_obj(d) for d in data[start_idx:end_idx]]
                )
                self.__collection.extend(downloaded_media)
        else:
            downloaded_media = await asyncio.gather(*[self.__get_media_obj(d) for d in data])

            self.__collection.extend(downloaded_media)

        return self.__collection

    @classmethod
    async def __get_media_obj(cls, data):
        return IGMediaObject(data)

    @property
    def collection(self):
        return self.__collection



# Generated by Django 4.0 on 2022-08-22 04:42

import encrypted_model_fields.fields
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = []

    operations = [
        migrations.CreateModel(
            name="InstagramAccount",
            fields=[
                (
                    "account_id",
                    models.PositiveBigIntegerField(primary_key=True, serialize=False),
                ),
                ("access_token", encrypted_model_fields.fields.EncryptedCharField()),
                ("authorized_on", models.DateTimeField(auto_now_add=True)),
            ],
            options={
                "verbose_name_plural": "Instagram Accounts",
                "db_table": "instagram_account",
            },
        ),
    ]



# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['instagram_sync',
 'instagram_sync.contrib',
 'instagram_sync.contrib.django',
 'instagram_sync.contrib.django.migrations',
 'instagram_sync.core',
 'instagram_sync.core.graph_api']

package_data = \
{'': ['*'], 'instagram_sync.contrib.django': ['templates/instagram_sync/*']}

install_requires = \
['requests>=2.28.1,<3.0.0']

extras_require = \
{'django': ['Django>=3.0', 'django-encrypted-model-fields>=0.6.5,<0.7.0']}

setup_kwargs = {
    'name': 'instagram-sync',
    'version': '0.1.2',
    'description': '',
    'long_description': None,
    'author': 'MJ Hardin',
    'author_email': 'mhardinla@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'extras_require': extras_require,
    'python_requires': '>=3.9,<4.0',
}


setup(**setup_kwargs)
